bad_keywords_list=[
'AMPLIST',
'AMPNAME',
'ASCLDMED',
'ASCLDMEN',
'ASCLDMOD',
'ASCLDSTD',
'ASECA',
'ASECB',
'ASNERMED',
'ASNERMEN',
'ASNERMOD',
'ASNERSTD',
'ASNESMED',
'ASNESMEN',
'ASNESMOD',
'ASNESSTD',
'ASNWRMED',
'ASNWRMEN',
'ASNWRMOD',
'ASNWRSTD',
'ASNWSMED',
'ASNWSMEN',
'ASNWSMOD',
'ASNWSSTD',
'ASSERMED',
'ASSERMEN',
'ASSERMOD',
'ASSERSTD',
'ASSESMED',
'ASSESMEN',
'ASSESMOD',
'ASSESSTD',
'ASSPRMED',
'ASSPRMEN',
'ASSPRMOD',
'ASSPRSTD',
'ASSPSMED',
'ASSPSMEN',
'ASSPSMOD',
'ASSPSSTD',
'ASSWRMED',
'ASSWRMEN',
'ASSWRMOD',
'ASSWRSTD',
'ASSWSMED',
'ASSWSMEN',
'ASSWSMOD',
'ASSWSSTD',
'BIAS0',
'BIAS1',
'BIASSEC',
'BITPIX',
'BONANGLE',
'BSECA',
'BSECB',
'CALFOCUS',
'CCD',
'CCDBIN1',
'CCDBIN2',
'CCDNICK',
'CCDSUM',
'CMMTOBS',
'CMMTSEQ',
'COMMENT',
'CONSWV',
'CSECA',
'CSECB',
'DARKCUR',
'DARKTIME',
'DATE',
'DDSEEMED',
'DDSEEMEN',
'DDSEEMOD',
'DDSEESTD',
'DECOFF',
'DEFOCUS',
'DETECTOR',
'DETSECA',
'DETSECB',
'DETSTAT',
'DETTEM',
'DOMEAZ',
'DOMELOUV',
'DOMESHUT',
'DSECA',
'DSECB',
'EXPMEAS',
'EXPREQ',
'EXPTYPE',
'EXTVER',
'FILTERID',
'FOCUSID',
'FOCUSPOS',
'FSAGAIN',
'FSAMAXZ',
'FSAMEANZ',
'FSAMINZ',
'FSANMOVE',
'FSARATE',
'FSASTATE',
'FSASTDVZ',
'FSATHRES',
'GAIN',
'GAINA',
'GAINB',
'GCOUNT',
'GUIDEC',
'GUIDEC1',
'GUIDEC2',
'GUIDECPM',
'GUIDEPM1',
'GUIDEPM2',
'GUIEQUI1',
'GUIEQUI2',
'GUIEQUIN',
'GUIFLUX',
'GUIFLUX1',
'GUIFLUX2',
'GUIFWHX',
'GUIFWHX1',
'GUIFWHX2',
'GUIFWHY',
'GUIFWHY1',
'GUIFWHY2',
'GUIMAGN',
'GUIMAGN1',
'GUIMAGN2',
'GUINAME',
'GUINAME1',
'GUINAME2',
'GUIOBJN',
'GUIOBJN1',
'GUIOBJN2',
'GUIPOSX1',
'GUIPOSX2',
'GUIPOSY1',
'GUIPOSY2',
'GUIPOSZ1',
'GUIPOSZ2',
'GUIRA',
'GUIRA1',
'GUIRA2',
'GUIRADE1',
'GUIRADE2',
'GUIRADEC',
'GUIRAPM',
'GUIRAPM1',
'GUIRAPM2',
'HA',
'HAEND',
'HISTORY',
'HSTTIME',
'IMAGESWV',
'INHERIT',
'INSTMODE',
'INSTRUME',
'INSTZDEC',
'INSTZRA',
'ISUGAIN',
'ISURATE',
'ISUSTATE',
'ISUSTDVX',
'ISUSTDVY',
'ISUTCSOR',
'LATITUDE',
'LONGITUD',
'MAXLIN',
'MAXLINA',
'MAXLINB',
'MIRRCOV',
'MOONALT',
'MOONANGL',
'MOONAZ',
'MOONPHAS',
'MOONUP',
'NAXIS',
'NAXIS1',
'NAXIS2',
'NGUIDER',
'NGUISTAR',
'OBJDEC',
'OBJDECPM',
'OBJEQUIN',
'OBJMAG',
'OBJRA',
'OBJRADEC',
'OBJRAPM',
'OBSERVER',
'OBSID',
'OBSTYPE',
'ORIGIN',
'PATHNAME',
'PCOUNT',
'PIXSCAL1',
'PIXSCAL2',
'PIXSIZE1',
'PIXSIZE2',
'PI_NAME',
'PRESSURE',
'QCOORD',
'QEPOINTS',
'QICLABEL',
'QICSEQID',
'QID',
'QOBITER',
'QOBLABEL',
'QOBSEQID',
'QOBSERVE',
'QOFFID',
'QOGLABEL',
'RAOFF',
'RASTER',
'RDNOISE',
'RDNOISEA',
'RDNOISEB',
'REGMODE',
'RELHUMID',
'REL_DATE',
'ROOTEND',
'ROTANGLE',
'SATURATE',
'SHUTCLOS',
'SHUTOPEN',
'SKYFLUX',
'SPBZPMED',
'SPBZPMEN',
'SPBZPMOD',
'SPBZPSTD',
'SPOZPMED',
'SPOZPMEN',
'SPOZPMOD',
'SPOZPSTD',
'SPVZPMED',
'SPVZPMEN',
'SPVZPMOD',
'SPVZPSTD',
'TCSAMODE',
'TCSAPDEC',
'TCSAPHA',
'TCSEDDEC',
'TCSEDHA',
'TCSENDEC',
'TCSENHA',
'TCSEPICS',
'TCSGEDEC',
'TCSGEHA',
'TCSGPSBC',
'TCSGPSTM',
'TCSINDEC',
'TCSINHA',
'TCSLST',
'TCSMJD',
'TCSMVDEC',
'TCSMVRA',
'TCSMVX',
'TCSMVY',
'TCSOBDEC',
'TCSOBHA',
'TCSRBUSS',
'TCSSEDEC',
'TCSSEHA',
'TEA2INCH',
'TEA2INEB',
'TEA6FOOT',
'TEALOWWS',
'TEAMIRCI',
'TEAMIRCO',
'TEANRLSB',
'TEAPMCLW',
'TEAPMSPN',
'TEAPMSPS',
'TEATOPOP',
'TEATOPWS',
'TEATRNGE',
'TEATRNGW',
'TEAWTHRT',
'TELALT',
'TELAZ',
'TELCONF',
'TELESCOP',
'TELFOCUS',
'TELSTAT',
'TEMPERAT',
'TESCONRM',
'TESHRSET',
'TESPIERN',
'TESPIERS',
'TESPMIRE',
'TESPMIRS',
'TESPMIRW',
'TESTSPHE',
'TESTTREM',
'TESTTRNM',
'TESTTRSE',
'TESTTRSH',
'TESTTRSL',
'TESTTRWM',
'TIMESYS',
'TSECA',
'TSECB',
'VENTL1',
'VENTL2',
'VENTL3',
'VENTL4',
'VENTL5',
'VENTL6',
'VENTMODE',
'VENTR1',
'VENTR2',
'VENTR3',
'VENTR4',
'VENTR5',
'VENTR6',
'WDSEEMED',
'WDSEEMEN',
'WDSEEMOD',
'WDSEESTD',
'WINDDIR',
'WINDSCR',
'WINDSPED',
'WMSEEMED',
'WMSEEMEN',
'WMSEEMOD',
'WMSEESTD',
'XPROBE',
'XTENSION',
'YPROBE',
'ZPROBE',
],
